@@ -1,6 +1,6 @@
 from matrx.actions.action import Action, ActionResult
 from matrx.actions.door_actions import OpenDoorAction, OpenDoorActionResult, CloseDoorAction, CloseDoorActionResult
 from matrx.actions.move_actions import MoveActionResult, MoveNorth, MoveNorthEast, MoveSouthEast, MoveSouth, \
-    MoveSouthWest, MoveWest, MoveNorthWest
+    MoveSouthWest, MoveWest, MoveNorthWest, MoveEast
 from matrx.actions.object_actions import RemoveObject, RemoveObjectResult, GrabObject, GrabObjectResult, \
     DropObject, DropObjectResult
