@@ -24,12 +24,11 @@ port = 3001
 
 # variables to be set by MATRX
 # states is a list of length 'current_tick' with a dictionary containing all states of that tick, indexed by agent_id
-states = {}
+states = []
 current_tick = 0
 tick_duration = 0.5
 grid_size = [1, 1]
 MATRX_info = {}
-nr_states_to_store = 5
 next_tick_info = {}
 add_message_to_agent = None
 received_messages = {}  # messages received via the api, intended for the Gridworld
@@ -66,33 +65,113 @@ def get_info():
     return jsonify(MATRX_info)
 
 
-@app.route('/get_latest_state_and_messages/<agent_id>', methods=['GET', 'POST'])
-def get_latest_state_and_messages(agent_id):
-    """ Provides all most recent information from MATRX: Both the state and messages from the latest
-    tick for one particular agent, as well as the current MATRX status (paused or not).
+@app.route('/get_latest_state_and_messages/', methods=['GET', 'POST'])
+def get_latest_state_and_messages():
+    """ Provides all most recent information from MATRX for 1 agent: The state from the latest tick, and any new
+    messages and chatrooms.
+
+    Parameters should be passed via GET URL parameters.
+
+    A combination of :func:`~matrx.api.api.get_latest_state` and :func:`~matrx.api.api.get_messages`. See those
+    two functions for their respective documentation.
 
     Parameters
     ----------
-    agent_id
-        The ID of the targeted agent
+    agent_id : (required GET URL parameter, default {})
+        The ID of the targeted agent. Only the state of that agent, and chatrooms in which that agent is part will be
+        sent.
+
+    chat_offsets : (optional GET URL parameter, default {})
+        It is not efficient to send every message every tick. With this offsets dict the requestee can
+        indicate for every chatroom, which messages they already have, such that only new messages can be sent.
+        The `offsets` URL parmaeter should be a dict with as keys the chatroom ID, and as values the message offset.
+        The message offset is the index of the message.
+        Example of a valid dict: {"0": "10", "3": "5"}.
+        This returns the message with index 10+ for the chatroom with ID 0 (global chat),
+        and messages with index 5+ for chatroom with ID 3.
 
     Returns
     -------
-        a dictionary containing the states under the "states" key, and the messages under the "messages" key.
+        A dictionary containing the states under the "states" key, and the chatrooms with messages under the
+         "chatrooms" key.
 
     """
+
+    # from GET requests fetch URL parameters
+    if request.method == "GET":
+        error_mssg = f"The /get_latest_state_and_messages/ API call only allows POST requests for MATRX Version 2.0.0 " \
+                     f"and higher. Please see https://matrx-software.com/docs/upgrading-matrx on how to upgrade."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
+
+    # For POST requests fetch json data
+    elif request.method == "POST":
+        data = request.json
+        agent_id = None if "agent_id" not in data else data['agent_id']
+        chat_offsets = None if "chat_offsets" not in data else data['chat_offsets']
+
+    else:
+        error_mssg = f"API call only allows POST requests."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
+
+    # agent_id is required
+    if not isinstance(agent_id, str):
+        error_mssg = f"Agent_id passed to /get_latest_state_and_messages API request is not of valid format: " \
+                     f"{agent_id}. Should be string."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
+
     # check for validity and return an error if not valid
     api_call_valid, error = check_states_API_request(ids=[agent_id])
     if not api_call_valid:
         print("api request not valid:", error)
         return abort(error['error_code'], description=error['error_message'])
 
-    # fetch states and messages
+    # fetch states, chatrooms and messages
     states_ = __fetch_states(current_tick, agent_id)
-    messages = gw_message_manager.fetch_messages(current_tick, current_tick, clean_input_ids(agent_id)[0])
-    chatrooms = gw_message_manager.fetch_chatrooms(clean_input_ids(agent_id)[0])
+    chatrooms, messages = get_messages(agent_id, chat_offsets)
 
-    return jsonify({"matrx_paused": matrx_paused, "states": states_, "messages": messages, "chatrooms": chatrooms})
+    return jsonify({"matrx_paused": matrx_paused, "states": states_, "chatrooms": chatrooms, "messages": messages})
+
+
+#
+# @app.route('/get_latest_state_and_messages/<agent_id>', methods=['GET', 'POST'])
+# def get_latest_state_and_messages(agent_id):
+#     """ Provides all most recent information from MATRX: Both the state and messages from the latest
+#     tick for one particular agent, as well as the current MATRX status (paused or not).
+#
+#     Parameters
+#     ----------
+#     agent_id
+#         The ID of the targeted agent
+#
+#     agent_id : (optional GET URL parameter, default None)
+#         Agent ID string that will make this function only return chatrooms of which that agent is part. Defaults to
+#         None, returning all chatsrooms and all chat messages.
+#
+#     offsets : (optional GET URL parameter, default {})
+#         GET URL parameter that should be a dict with as key the chatroom ID, and as value the message offset. E.g.
+#         {"0": "10", "3": "5"}. This returns the message with index 10+ for the chatroom with ID 0 (global chat),
+#         and messages with index 5+ for chatroom with ID 3.
+#
+#     Returns
+#     -------
+#         a dictionary containing the states under the "states" key, and the messages under the "messages" key.
+#
+#     """
+#     # TODO: deprecated
+#     # check for validity and return an error if not valid
+#     api_call_valid, error = check_states_API_request(ids=[agent_id])
+#     if not api_call_valid:
+#         print("api request not valid:", error)
+#         return abort(error['error_code'], description=error['error_message'])
+#
+#     # fetch states and chatrooms with messages
+#     states_ = __fetch_states(current_tick, agent_id)
+#     chatrooms = get_messages(request)
+#
+#     return jsonify({"matrx_paused": matrx_paused, "states": states_, "messages": {}, "chatrooms": chatrooms})
 
 
 #########################################################################
@@ -114,6 +193,9 @@ def get_states(tick):
         state for each agent existing in the simulation, indexed by their agent ID.
 
     """
+    # fetch URL parameters
+    agent_id = request.args.get("agent_id")
+    chat_offsets = request.args.get("chat_offsets")
 
     # check for validity and return an error if not valid
     api_call_valid, error = check_states_API_request(tick=tick)
@@ -171,9 +253,7 @@ def get_latest_state(agent_ids):
 
 @app.route('/get_filtered_latest_state/<agent_ids>', methods=['POST'])
 def get_filtered_latest_state(agent_ids):
-    """
-
-    """
+    """ Return a state from a set of agent IDs, filtered to only return the specified properties """
 
     # check for validity and return an error if not valid
     api_call_valid, error = check_states_API_request(tick=current_tick)
@@ -203,132 +283,94 @@ def get_filtered_latest_state(agent_ids):
 #########################################################################
 # MATRX fetch messages api calls
 #########################################################################
-@app.route('/get_messages/<tick>', methods=['GET', 'POST'])
-def get_messages(tick):
-    """ Provides the messages of all agents from tick `tick` onwards to current tick. Also returns
-    the chatrooms at the latest tick.
+@app.route('/get_messages/', methods=['GET', 'POST'])
+def get_messages_apicall():
+    """ Returns chatrooms and chat messages for one agent, or all agents.
+
+    Per chatroom, an offset can be passed from which will only return messages with a higher index than that
+     offset.
+
+    Parameters should be passed via GET URL parameters.
 
     Parameters
     ----------
-    tick
-        integer indicating from which tick onwards to send the messages.
+    agent_id : (optional URL parameter, default None)
+        Agent ID string that will make this function only return chatrooms of which that agent is part. Defaults to
+        None, returning all chatsrooms and all chat messages.
 
+    chat_offsets : (optional URL parameter, default None)
+        It is not efficient to send every message every tick. With this chat_offsets dict the requestee can
+        indicate for every chatroom, which messages they already have, such that only new messages can be sent.
+        The `chat_offsets` URL parmaeter should be a dict with as keys the chatroom ID, and as values the message offset.
+        The message offset is the index of the message.
+        Example of a valid dict: {"0": "10", "3": "5"}.
+        This returns the message with index 10+ for the chatroom with ID 0 (global chat),
+        and messages with index 5+ for chatroom with ID 3.
     Returns
     -------
-        Returns a dictionary containing `messages` and `chatrooms`. The chatrooms subdictionary
-        contains all accessible chatrooms at the latest tick. The messages subdictionary contains
-        all messages for tick `tick` to the current tick,
-        subdivided under `global`, `team`, and `private`.
+        Returns a dictionary with chatrooms and per chatroom a list per with messages.
+        The dict is in the shape of: {chatroom_ID: [Message1, Message2, ..], chatroom_ID2 : ....}
+
         Also see the documentation of the
         :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_messages` and
         :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_chatrooms` functions.
     """
 
-    # check for validity and return an error if not valid
-    api_call_valid, error = check_messages_API_request(tick=tick)
-    if not api_call_valid:
-        print("api request not valid:", error)
-        return abort(error['error_code'], description=error['error_message'])
+    # from GET requests fetch URL parameters
+    if request.method == "GET":
+        error_mssg = f"The /get_messages/ API call only allows POST requests for MATRX Version 2.0.0 and higher. " \
+                     f"Please see https://matrx-software.com/docs/upgrading-matrx on how to upgrade."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
 
-    messages = gw_message_manager.fetch_messages(int(tick), current_tick)
-    chatrooms = gw_message_manager.fetch_chatrooms()
+    # For POST requests fetch json data
+    elif request.method == "POST":
+        data = request.json
+        agent_id = None if "agent_id" not in data else data['agent_id']
+        chat_offsets = None if "chat_offsets" not in data else data['chat_offsets']
 
-    return jsonify({"messages": messages, "chatrooms": chatrooms})
+    else:
+        error_mssg = f"API call only allows POST requests."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
+
+    chatrooms, messages = get_messages(agent_id, chat_offsets)
+
+    return jsonify({"chatrooms": chatrooms, "messages": messages})
 
 
-@app.route('/get_messages/<tick>/<agent_id>', methods=['GET', 'POST'])
-def get_messages_specific_agent(tick, agent_id):
-    """ Provides all messages either send by or addressed to `agent_id`, from tick `tick` onwards.
+def get_messages(agent_id, chat_offsets):
+    """ See :func:`~matrx.api..message_manager.get_messages` """
 
-    Parameters
-    ----------
-    tick
-        integer indicating which ticks [`tick`,current_tick] (including `tick` and current_tick)
-        the messages should be fetched from.
-    agent_id
-        The `agent_id` of the agent of whom the messages should be fetched.
-        All fetched messages are either sent to or by the agent with agent ID `agent_id`.
+    # validate agent_id: None or str
+    if not isinstance(agent_id, str) and not agent_id is None:
+        error_mssg = f"Agent_id passed to /get_messages API request is not of valid format: {agent_id}. " \
+                     f"Should be string or not passed."
+        print("api request not valid:", error_mssg)
+        return abort(400, description=error_mssg)
 
-    Returns
-    -------
-        Returns a dictionary containing `messages` and `chatrooms`. The chatrooms subdictionary
-        contains all accessible chatrooms at the latest tick. The messages subdictionary contains
-        all messages sent or received by `agent_id`,
-        subdivided under `global`, `team`, and `private`.
-        Also see the documentation of the
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_messages` and
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_chatrooms` functions.
+    if not isinstance(chat_offsets, dict) and not chat_offsets is None:
+        error_mssg = f"Chatroom message chat_offsets passed to /get_messages API request is not of valid format: " \
+                     f"{chat_offsets}. Should be a dict or not passed."
+        print("api request not valid:", error_mssg)
+        print(chat_offsets)
+        return abort(400, description=error_mssg)
 
-    """
-    # check for validity and return an error if not valid
-    api_call_valid, error = check_messages_API_request(tick=tick, agent_id=agent_id)
-    if not api_call_valid:
-        print("api request not valid:", error)
-        return abort(error['error_code'], description=error['error_message'])
+    # fetch chatrooms with messages for the passed agent_id and return it
+    chatrooms = gw_message_manager.fetch_chatrooms(agent_id=agent_id)
+    messages = gw_message_manager.fetch_messages(agent_id=agent_id, chatroom_mssg_offsets=chat_offsets)
 
-    messages = gw_message_manager.fetch_messages(int(tick), current_tick, clean_input_ids(agent_id)[0])
-    chatrooms = gw_message_manager.fetch_chatrooms(clean_input_ids(agent_id)[0])
-
-    return jsonify({"messages": messages, "chatrooms": chatrooms})
+    return chatrooms, messages
 
 
-@app.route('/get_latest_messages', methods=['GET', 'POST'])
-def get_latest_messages():
-    """ Provides all messages of the latest tick.
-
-    Returns
-        Returns a dictionary containing `messages` and `chatrooms`. The chatrooms subdictionary
-        contains all accessible chatrooms at the latest tick. The messages subdictionary contains
-        all messages of the latest tick, subdivided under `global`, `team`, and `private`.
-        Also see the documentation of the
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_messages` and
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_chatrooms` functions.
-    -------
-
-    """
-    # check for validity and return an error if not valid
-    api_call_valid, error = check_messages_API_request(tick=current_tick)
-    if not api_call_valid:
-        print("api request not valid:", error)
-        return abort(error['error_code'], description=error['error_message'])
-
-    messages = gw_message_manager.fetch_messages(current_tick, current_tick)
-    chatrooms = gw_message_manager.fetch_chatrooms()
-
-    return jsonify({"messages": messages, "chatrooms": chatrooms})
-
-
-@app.route('/get_latest_messages/<agent_id>', methods=['GET', 'POST'])
-def get_latest_messages_specific_agent(agent_id):
-    """ Provides the messages of the latest tick either sent by or addressed to `agent_id`.
-
-    Parameters
-    ----------
-    agent_id
-        The `agent_id` of the agent of whom the messages should be fetched.
-        All fetched messages are either sent to or by the agent with agent ID `agent_id`.
-
-    Returns
-    -------
-        Returns a dictionary containing `messages` and `chatrooms`. The chatrooms subdictionary
-        contains all accessible chatrooms at the latest tick, set to or by the agent with
-        ID `agent_id`. The messages subdictionary contains
-        all messages of the latest tick, subdivided under `global`, `team`, and `private`.
-        Also see the documentation of the
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_messages` and
-        :func:`~matrx.utils.message_manager.MessageManager.MyClass.fetch_chatrooms` functions.
-
-    """
-    # check for validity and return an error if not valid
-    api_call_valid, error = check_messages_API_request(tick=current_tick, agent_id=agent_id)
-    if not api_call_valid:
-        print("api request not valid:", error)
-        return abort(error['error_code'], description=error['error_message'])
-
-    messages = gw_message_manager.fetch_messages(current_tick, current_tick, clean_input_ids(agent_id)[0])
-    chatrooms = gw_message_manager.fetch_chatrooms(clean_input_ids(agent_id)[0])
-
-    return jsonify({"messages": messages, "chatrooms": chatrooms})
+# @app.route('/get_messages/<tick>/<agent_id>', methods=['GET'])
+# @app.route('/get_messages/<tick>', methods=['GET', 'POST'])
+# @app.route('/get_messages/<agent_id>', methods=['GET', 'POST'])
+# @app.route('/get_latest_messages', methods=['GET', 'POST'])
+# @app.route('/get_latest_messages/<agent_id>', methods=['GET', 'POST'])
+# def deprecated_get_messages(agent_id):
+#     #TODO: deprecated
+#     return jsonify(True)
 
 
 #########################################################################
@@ -393,6 +435,8 @@ def send_message():
     # fetch the data
     data = request.json
 
+    print("Received message to send with data:", data)
+
     # check that all required parameters have been passed
     required_params = ("content", "sender", "receiver")
     if not all(k in data for k in required_params):
@@ -435,6 +479,12 @@ def fetch_context_menu_of_self():
     if agent_id_who_clicked not in gw.registered_agents.keys() and agent_id_who_clicked != "god":
         return return_error(code=400, message=f"Agent with ID {agent_id_who_clicked} does not exist.")
 
+    # check if it is a human agent
+    if agent_id_who_clicked in gw.registered_agents.keys() and \
+            not gw.registered_agent_keys[agent_id_who_clicked].is_human_agent:
+        return return_error(code=400, message=f"Agent with ID {agent_id_who_clicked} is not a human agent and thus does"
+                                              f" not have a context_menu_of_self() function.")
+
     # ignore if called from the god view
     if agent_id_who_clicked.lower() == "god":
         return return_error(code=400,
@@ -499,7 +549,7 @@ def fetch_context_menu_of_other():
 def send_message_pickled():
     """ This function makes it possible to send a custom message to a MATRX agent via the API as a jsonpickle object.
     For instance, sending a custom message when a context menu option is clicked.
-    The pre-formatted CustomMessage instance an be jsonpickled and sent via the API.
+    The pre-formatted CustomMessage instance can be jsonpickled and sent via the API.
     This API call can handle that request and send the CustomMessage to the MATRX agent
 
         Returns
@@ -742,7 +792,7 @@ def check_states_API_request(tick=None, ids=None, ids_required=False):
             # e.g. "god"), or a list of ' f'IDs(string) for requesting states of multiple agents'}
 
         # check if the api was reset during this time
-        if len(states) == 0:
+        if len(states) is 0:
             return False, {'error_code': 400,
                            'error_message': f'api is reconnecting to a new world'}
 
@@ -775,12 +825,6 @@ def check_input(tick=None, ids=None):
                            'error_message': f'Indicated tick does not exist, has to be in range 0 - {current_tick}, '
                                             f'but is {tick}'}
 
-        # check if the tick was stored
-        if tick not in states.keys():
-            return False, {'error_code': 400,
-                           'error_message': f'Indicated tick {tick} is not stored, only the {nr_states_to_store} ticks '
-                                            f'are stored that occurred before the current tick {current_tick}'}
-
     return True, None
 
 
@@ -801,16 +845,11 @@ def __fetch_states(tick, ids=None):
         specified in `agent_ids`, indexed by their agent ID.
 
     """
-    check_passed, error_message = check_input(tick)
-    if not check_passed:
-        return False, error_message
     tick = int(tick)
 
     # return all states
     if ids is None:
-        # Get the right states based on the tick and return them
-        return_states = [state for t, state in states.items() if t >= tick <= current_tick]
-        return return_states
+        return states[tick:]
 
     # convert ints to lists so we can use 1 uniform approach
     try:
@@ -825,7 +864,8 @@ def __fetch_states(tick, ids=None):
 
         # add each agent's state for this tick
         for agent_id in ids:
-            states_this_tick[agent_id] = states[t][agent_id]
+            if agent_id in states[t]:
+                states_this_tick[agent_id] = states[t][agent_id]
 
         # save the states of all filtered agents for this tick
         filtered_states.append(states_this_tick)
@@ -910,7 +950,7 @@ def __reorder_state(state):
     # loop through all objects in the state
     for objID, obj in state.items():
 
-        if objID != "World":
+        if objID is not "World":
             # make the sense capability JSON serializable
             if "sense_capability" in obj:
                 new_state[objID]["sense_capability"] = str(obj["sense_capability"])
@@ -955,22 +995,13 @@ def next_tick():
     -------
     """
     # save the new general info
-    global MATRX_info, next_tick_info, states, nr_states_to_store, current_tick, stored_state_ticks
+    global MATRX_info, next_tick_info
     MATRX_info = copy.copy(next_tick_info)
     next_tick_info = {}
     # print("Next ticK:", MATRX_info);
 
     # publicize the states of the previous tick
-    states[current_tick] = copy.copy(temp_state)
-
-    # Limit the states stored
-    if len(states) > nr_states_to_store:
-        stored_ticks = list(states.keys())
-        forget_from = current_tick - nr_states_to_store
-        for tick in stored_ticks:
-            if tick <= forget_from:
-                states.pop(tick)
-        print(len(states))
+    states.append(copy.copy(temp_state))
 
 
 def pop_userinput(agent_id):
@@ -992,14 +1023,13 @@ def pop_userinput(agent_id):
 
 def reset_api():
     """ Reset the MATRX api variables """
-    global temp_state, userinput, matrx_paused, matrx_done, states, current_tick, tick_duration, grid_size, \
-        nr_states_to_store
+    global temp_state, userinput, matrx_paused, matrx_done, states, current_tick, tick_duration, grid_size
     global MATRX_info, next_tick_info, received_messages, current_world_ID
     temp_state = {}
     userinput = {}
     matrx_paused = False
     matrx_done = False
-    states = {}
+    states = []
     current_tick = 0
     tick_duration = 0.0
     grid_size = [1, 1]
@@ -1007,7 +1037,6 @@ def reset_api():
     next_tick_info = {}
     received_messages = {}
     current_world_ID = False
-    nr_states_to_store = 5
 
 
 def register_world(world_id):
