@@ -170,18 +170,15 @@ function populate_agent_menu(state) {
  ********************************************************************/
 var showing_chat = false;
 // which chatrooms are added and have a tab
-var chatrooms_added = {
-    "global": null,
-    "team": [],
-    "private": []
-};
+var active_chatrooms = [];
 // all the possible chatrooms
 var all_chatrooms = {};
+// offsets of the latest messages for each chatroom
+var chat_offsets = {};
 // the currently opened chatroom
-var current_chatwindow = {
-    'name': 'global',
-    'type': 'global'
-};
+var current_chatwindow = {};
+// our database with messages, indexed by chat room name
+var messages = {};
 
 /*
  * Add a key listener to the input text box, such that the message will be sent when the user
@@ -199,9 +196,12 @@ if (chat_input_box != null) {
 
 
 /*
- * Load the chat rooms when the user clicks the "+" chat room button
+ * Show the chatrooms which the user can open when the user clicks the "+"
+ * chatroom button
  */
 function populate_new_chat_dropdown(matrx_chatrooms) {
+//    console.log("Repopulating new chat dropdown");
+    // add to our chatroom database
     all_chatrooms = matrx_chatrooms;
 
     // get the dropdown from html
@@ -212,58 +212,78 @@ function populate_new_chat_dropdown(matrx_chatrooms) {
         dropdown.removeChild(dropdown.firstChild);
     }
 
-    // get chatroom options
-    var private_chatroom_options = all_chatrooms['private'].filter(n => !chatrooms_added['private'].includes(n));
-    var team_chatroom_options = all_chatrooms['team'].filter(n => !chatrooms_added['team'].includes(n));
+    // check if there are any new chatrooms that have to be added to the
+    // dropdown chatroom list
+    Object.keys(matrx_chatrooms).forEach(function(chatroom_ID) {
 
-    // add private chat room options
-    private_chatroom_options.forEach(function(chat_name) {
+        // skip already active chatrooms
+        if (active_chatrooms.includes(chatroom_ID)) {
+            return;
+        }
 
-        // create a new dropdown item and add the preview and agent name
+        chatroom = matrx_chatrooms[chatroom_ID];
+
+        // create a new dropdown item and add the chatroom name
         var list_item = document.createElement('a');
         list_item.classList.add('dropdown-item');
         list_item.classList.add('new_chat_option');
-        list_item.appendChild(document.createTextNode(chat_name)); // name
-        list_item.chat_name = chat_name;
-        list_item.room_type = "private";
-        list_item.addEventListener('click', function(event) {
-            add_chatroom(event.currentTarget.chat_name, event.currentTarget.room_type);
-            open_chatroom(event.currentTarget.chat_name, event.currentTarget.room_type);
-
-            // repopulate the new chat room dropdown
-            populate_new_chat_dropdown(all_chatrooms);
-        });
-
-        // add the agent to the dropdown list
-        dropdown.append(list_item);
-    });
-
-    // add team chat room options
-    team_chatroom_options.forEach(function(chat_name) {
-
-        // create a new dropdown item and add the preview and agent name
-        var list_item = document.createElement('a');
-        list_item.classList.add('dropdown-item');
-        list_item.classList.add('new_chat_option');
-        list_item.appendChild(document.createTextNode(chat_name)); // name
-        list_item.chat_name = chat_name;
-        list_item.room_type = "team";
+        list_item.id = "new_chatroom_option_" + chatroom_ID;
+        list_item.chat_name = chatroom['name'];
+        list_item.chatroom_display_name = get_chatroom_display_name(chatroom);
+        list_item.appendChild(document.createTextNode(list_item.chatroom_display_name)); // name
+        list_item.chatroom_ID = chatroom_ID;
+        list_item.room_type = chatroom['type'];
 
         // on click add this chat room
         list_item.addEventListener('click', function(event) {
-            add_chatroom(event.currentTarget.chat_name, event.currentTarget.room_type);
-            open_chatroom(event.currentTarget.chat_name, event.currentTarget.room_type);
 
-            // repopulate the new chat room dropdown
-            populate_new_chat_dropdown(all_chatrooms);
+            add_chatroom(event.currentTarget.chat_name, event.currentTarget.chatroom_display_name,
+                event.currentTarget.chatroom_ID, event.currentTarget.room_type);
+            open_chatroom(event.currentTarget.chatroom_display_name, event.currentTarget.chatroom_ID,
+                event.currentTarget.room_type);
+
+            // remove this chat from the new chat dropdown
+            document.getElementById(event.currentTarget.id).remove();
         });
 
-        // add the agent to the dropdown list
+        // add the chatroom to the dropdown list
         dropdown.append(list_item);
+//        console.log("added item to list");
     });
 }
 
 
+/*
+ * Get the name that should be displayed for a chatroom in the dropdown / chatroom list
+ */
+function get_chatroom_display_name(chatroom) {
+    // by default the display name is the same as the chatroom name
+    var display_name = chatroom['name'];
+
+    // special care for the displayed name of private chats. These chatroom names are in the
+    // form of agentID1__agentID2. However that is not a very useful or nice chatroom name.
+    // So we remove our own agent ID from the name
+    if (chatroom['type'] == "private" && !(lv_agent_id == "god")) {
+        // split the chatroom name at the double underscore agent ID divider
+        var chat_agent_IDs = chatroom['name'].split("__");
+
+        // agents can send messages to themselves, display it as such
+        if (chat_agent_IDs[0] == chat_agent_IDs[1]) {
+            return "messages to self"
+        }
+
+        // in a private chat with someone else, only display the ID of the other
+        chat_agent_IDs.forEach(function(id) {
+            if (id != lv_agent_id) {
+                display_name = id;
+            }
+        });
+    }
+
+    return display_name
+}
+
+
 /*
  * Hide or show the chat, when the chat button is pressed in the toolbar
  */
@@ -277,10 +297,17 @@ function chatToggle() {
 }
 
 /*
- * Add a chat room to the GUI
+ * Add the chatroom to the visible chatrooms
  */
-function add_chatroom(chat_name, room_type, set_active = true) {
-    console.log("Adding", room_type, "chatroom:", chat_name);
+function add_chatroom(chat_name, chatroom_display_name, chatroom_ID, room_type, set_active = true) {
+//    console.log("Adding", room_type, "chatroom:", chat_name, "chatroom_display_name:", chatroom_display_name,
+//        "chatID:", chatroom_ID);
+
+    // add a new list that will keep track of the messages of this chatroom
+    messages[chatroom_ID] = [];
+
+    // set the offset to the first message
+    chat_offsets[chatroom_ID] = null;
 
     // set the previously open chatroom to inactive
     if (set_active) {
@@ -290,41 +317,45 @@ function add_chatroom(chat_name, room_type, set_active = true) {
         }
     }
 
-    var chatrooms = document.getElementById("chat_rooms_list");
-    var global_chatroom = document.getElementById("chatroom_global");
+    var chatrooms = document.getElementById("chatrooms_list");
+    var new_chat_button = document.getElementById("new_chat_button");
 
     // create a new item and add the chatroom to the list
     var new_chatroom = document.createElement('div');
-    new_chatroom.id = "chatroom_" + chat_name;
+    new_chatroom.chatroom_ID = chatroom_ID;
+    new_chatroom.id = "chatroom_listitem_" + chatroom_ID;
     new_chatroom.room_type = room_type;
+    new_chatroom.name = chat_name;
+    new_chatroom.chatroom_display_name = chatroom_display_name;
     new_chatroom.classList.add('contact');
     if (set_active) {
         new_chatroom.classList.add('contact_active'); // set the clicked chatroom to active
     }
-    new_chatroom.appendChild(document.createTextNode(chat_name)); // name
+    new_chatroom.appendChild(document.createTextNode(chatroom_display_name)); // name
 
     // add a hidden (by default) notification circle, for when new messages have arrived in this
     // chat room while it is unopened
-    var chat_room_notification = document.createElement('span');
-    chat_room_notification.className = "chat-notification";
-    chat_room_notification.id = "chatroom_" + chat_name + "_notification";
-    chat_room_notification.style.display = "none";
-    new_chatroom.appendChild(chat_room_notification);
+    var chatroom_notification = document.createElement('span');
+    chatroom_notification.className = "chat-notification";
+    chatroom_notification.id = "chatroom_" + chatroom_ID + "_notification";
+    chatroom_notification.style.display = "none";
+    new_chatroom.appendChild(chatroom_notification);
 
     // When the user clicks on an added chat room, open it and display the messages
     new_chatroom.addEventListener('click', chatroom_click);
 
     // add the new chatroom before the global chatroom
-    chatrooms.insertBefore(new_chatroom, global_chatroom);
+    chatrooms.insertBefore(new_chatroom, new_chat_button);
 
     // note that we opened this chatroom
-    chatrooms_added[room_type].push(chat_name);
+    active_chatrooms.push(chatroom_ID);
 }
 
 /*
  * The user clicked on a chatroom, set it as a active and open the chatroom
  */
 function chatroom_click(event) {
+
     // set the previously open chatroom to inactive
     var active_contacts = document.getElementsByClassName("contact_active");
     if (active_contacts.length > 0) {
@@ -335,49 +366,57 @@ function chatroom_click(event) {
     event.currentTarget.className = "contact contact_active";
 
     // open the chatroom
-    open_chatroom(event.currentTarget.id, event.currentTarget.room_type);
+    open_chatroom(event.currentTarget.chatroom_display_name, event.currentTarget.chatroom_ID,
+        event.currentTarget.room_type);
 }
 
 
 /*
  * Open a chat room and display the messages.
  */
-function open_chatroom(chat_room, chat_room_type) {
+function open_chatroom(chatroom_display_name, chatroom_ID, chatroom_type) {
     // cleanup arguments
-    if (chat_room_type == null) {
-        chat_room_type = "global";
+    if (chatroom_type == null) {
+        chatroom_type = "global";
     }
-    chat_room = chat_room.replace("chatroom_", "");
 
     // set the chat room as selected
     current_chatwindow = {
-        'name': chat_room,
-        'type': chat_room_type
+        'chatroom_ID': chatroom_ID,
+        'name': chatroom_display_name,
+        'type': chatroom_type,
+        'receiver': null // by default null = addressed to everyone
     };
 
-    console.log("Opening", chat_room_type, "chatroom: ", chat_room);
+    // set the receiver for team and private messages
+    if (chatroom_type == 'private' || chatroom_type == 'team') {
+        current_chatwindow['receiver'] = chatroom_display_name;
+    }
+
+//    console.log("Opening", chatroom_type, "chatroom: ", all_chatrooms[chatroom_ID]['name'], " displayed as ",
+//            chatroom_display_name, " with ID ", chatroom_ID);
 
     // hide the notification of new messages for our goal chat room
-    document.getElementById("chatroom_" + chat_room + "_notification").style.display = "none";
+    document.getElementById("chatroom_" + chatroom_ID + "_notification").style.display = "none";
 
-    // remove old messages
+    // remove messages from the old chatroom
     var mssgs_container = document.getElementById("messages");
     while (mssgs_container.firstChild) {
         mssgs_container.removeChild(mssgs_container.firstChild);
     }
 
-    // display messages of this chat room, if any
-    if (Object.keys(messages).includes(chat_room)) {
-        messages[chat_room].forEach(function(message) {
-            add_message(chat_room, message, chat_room_type);
+    // display messages of this chatroom, if any
+    if (Object.keys(messages).includes(chatroom_ID)) {
+        messages[chatroom_ID].forEach(function(message) {
+            add_message(chatroom_ID, message);
         });
     }
 }
 
 /*
- * Add an individual message to the specified chat room
+ * Add 1 message to the currently opened chatroom
  */
-function add_message(chat_room, mssg, type) {
+function add_message(chatroom_ID, mssg) {
     // cleanup and validate the input
     mssg_content = mssg.content.trim();
     if (!mssg_content || mssg_content.length === 0) {
@@ -391,21 +430,18 @@ function add_message(chat_room, mssg, type) {
     if (mssg.from_id != lv_agent_id) {
         div.className = "message_other";
 
-        // display the sender name if it is a team or global chat message from someone else
-        if (type == "global" || type == "team") {
-            console.log("adding sender");
-            var mssg_sender = document.createElement('span');
-            mssg_sender.className = "chat-mssg-sender";
-            mssg_sender.appendChild(document.createTextNode(mssg.from_id + ": "));
-            div.appendChild(mssg_sender);
-        }
+        // display the sender name
+        // console.log("adding sender");
+        var mssg_sender = document.createElement('span');
+        mssg_sender.className = "chat-mssg-sender";
+        mssg_sender.appendChild(document.createTextNode(mssg.from_id + ": "));
+        div.appendChild(mssg_sender);
     }
 
-    // add the message content
+    // add the message text to the message div
     div.appendChild(document.createTextNode(mssg_content));
 
-
-    // add the message
+    // add the message div
     var mssgs_container = document.getElementById("messages");
     mssgs_container.appendChild(div);
 
@@ -436,11 +472,8 @@ function send_message(event) {
     // empty input field
     document.getElementById("chat_form_input").value = null;
 
-    // format receiver ID
-    var receiver = current_chatwindow['name'] == "global" ? null : current_chatwindow['name'];
-
     // send message to MATRX
-    data = {"content":user_message, "sender": lv_agent_id, "receiver": receiver}
+    data = {"content":user_message, "sender": lv_agent_id, "receiver": current_chatwindow['receiver']}
     console.log("Sending message to matrx:", data);
     send_matrx_api_post_message(matrx_send_message_url, data);
 }
@@ -452,12 +485,9 @@ function send_message(event) {
  */
 function reset_chat() {
     // remove the chat room buttons
-    chatrooms_added['team'].forEach(function(chat_room) {
-        document.getElementById("chatroom_" + chat_room).remove();
-    })
-    chatrooms_added['private'].forEach(function(chat_room) {
-        document.getElementById("chatroom_" + chat_room).remove();
-    })
+    active_chatrooms.forEach(function(chatroom_ID) {
+        document.getElementById("chatroom_" + chatroom_ID).remove();
+    });
 
     // remove old messages
     var mssgs_container = document.getElementById("messages");
@@ -466,20 +496,96 @@ function reset_chat() {
     }
 
     // reset the vars
-    chatrooms_added = {
-        "global": null,
-        "team": [],
-        "private": []
-    };
+    active_chatrooms = [];
     all_chatrooms = {};
-    current_chatwindow = {
-        'name': 'global',
-        'type': 'global'
-    };
+    chat_offsets = {};
+    current_chatwindow = {};
     messages = {};
 }
 
 
+/*
+ * Process the object containing messages of various types, received from MATRX, and process them
+ * into individual messages.
+ */
+function process_messages(new_messages, chatrooms) {
+    // the messages are in a object with as key the chatroom ID in which they belong
+    // Process the new messages for each chatroom
+    Object.keys(new_messages).forEach(function(chatroom_ID) {
+
+        // skip any chatrooms without messages
+        if (new_messages[chatroom_ID].length == 0) {
+
+            // repopulate the chatroom dropdown if it is new
+            if (!Object.keys(all_chatrooms).includes(chatroom_ID)) {
+                populate_new_chat_dropdown(chatrooms);
+            }
+            // skip any further processing
+            return
+        }
+
+        // add the chatroom to the active chat list if it is new and contains messages
+        if (!active_chatrooms.includes(chatroom_ID)) {
+            // fetch the chatroom data
+            chatroom = chatrooms[chatroom_ID];
+//            console.log("adding chatroom", chatroom);
+
+            // add the chatroom to our GUI (but don't show it yet)
+            var chatroom_display_name = get_chatroom_display_name(chatroom);
+            add_chatroom(chatroom['name'], chatroom_display_name, chatroom_ID, chatroom['type'], set_active=false);
+
+            // repopulate the chatroom dropdown
+            populate_new_chat_dropdown(chatrooms);
+//            console.log("all chatrooms:", all_chatrooms);
+        }
+
+        // fetch the messages for this chatroom
+        new_chatroom_mssgs = new_messages[chatroom_ID]
+
+        // the chatroom is currently active, so add and show the new messages
+        if (current_chatwindow['chatroom_ID'] == chatroom_ID) {
+
+            // add each message to our database and the GUI
+            new_chatroom_mssgs.forEach(function(mssg) {
+                mssg = jQuery.parseJSON(mssg);
+                chat_offsets[chatroom_ID] = mssg['chat_mssg_count']; // memorize the last mssg index
+                messages[chatroom_ID].push(mssg); // add to db
+                add_message(chatroom_ID, mssg); // add to GUI
+//                console.log("Adding message:", mssg, " type ", all_chatrooms[chatroom_ID]['type'], " chatroom ", all_chatrooms[chatroom_ID]['name']);
+//                console.log("Latest message offset is:", mssg['chat_mssg_count']);
+            });
+
+
+        // only add messages to our database but don't show
+        } else {
+            new_chatroom_mssgs.forEach(function(mssg) {
+                mssg = jQuery.parseJSON(mssg);
+                chat_offsets[chatroom_ID] = mssg['chat_mssg_count']; // memorize the last mssg index
+                messages[chatroom_ID].push(mssg); // add to db
+//                console.log("Adding message:", mssg, " type ", all_chatrooms[chatroom_ID]['type'], " chatroom ", all_chatrooms[chatroom_ID]['name']);
+//                console.log("Latest message offset (hidden) is:", mssg['chat_mssg_count']);
+            })
+            if (new_chatroom_mssgs.length != 0 ) {
+                // show the notification for the chat room
+                document.getElementById("chatroom_" + chatroom_ID + "_notification").style.display = "inline-block";
+            }
+        }
+    });
+
+    // if there is no chat active, set the last chat to active
+    if (jQuery.isEmptyObject(current_chatwindow) && active_chatrooms.length != 0) {
+
+        // fetch the last activated chatroom
+        chatroom_ID = active_chatrooms[active_chatrooms.length - 1]
+        chatroom = all_chatrooms[chatroom_ID];
+        // open it
+        open_chatroom(get_chatroom_display_name(chatroom), chatroom_ID, chatroom['type'])
+        // set it to active
+        document.getElementById("chatroom_listitem_" + chatroom_ID).className = "contact contact_active";
+    }
+
+}
+
 
 /*********************************************************************
  * Drawing tools
